%{
  Idealised numerical model for equilibrium shapes of tidal sandbanks under sediment-scarce conditions (1D)
  Copyright (C) 2023  Thomas van Veelen. Email: t.j.vanveelen@utwente.nl

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%}

%% Sandbanks_with_nonerodible layer (master)
% This script simulates the morphodynamic evolution of a tidal sandbank in
% sediment-scarce conditions. The model is idealized, 1D, and sediment
% scarcity is modeled by a non-erodible layer.This script is the master
% file. It loads variables, defines topography, sets up the parralel
% computation, calls the model and saves the results. For a full
% description of the input and output, see the README file.
%

clc, clear, close all

%% License
disp('Idealised numerical model for equilibrium shapes of tidal sandbanks under sediment-scarce conditions (1D).  Copyright (C) 2023  Thomas van Veelen. This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions ')

%% Path
addpath('subfunctions')

%% read input
inp = readtable('BankwithRock_runs.csv');

Casename = inp.Case;
Hstar = inp.Hstar;
Dsand = inp.Dsand;
Hrockstar = (1+Dsand).* Hstar;
drockstar = inp.delta .* Hstar;
hinit = inp.hinit;
topo_mode = inp.topo_mode;
Ldomstar = inp.Ldomstar;
domfac = inp.domfac;
lat = inp.lat;
semastar = [inp.Um0 inp.Um2 inp.Um4];
Uwstar = inp.Uwstar;
sigmastar = inp.sigmastar;
cd = inp.cd;
lambdastartilde = inp.lambdastartilde;
alphasstar = inp.alphasstar;
gammadeposition = inp.gammadeposition;
P = inp.P;
jdt = inp.jdt;
Mnonlin = inp.M;
dtmor = inp.dtmor;
tmormax = inp.tmormax;
EQcrit = inp.EQcrit;
Vnl0crit = inp.Vnl0crit;
Saveswitch = inp.Saveswitch;

nruns = numel(Casename);
Lfgm = nan(nruns,1);        % m; length of fastest growing mode
thetafgm = Lfgm;            % deg; Angle of the fastest growing mode
cfgm = Lfgm;                % m/yr; migration rate of the fastest groing mode
hhat = cell(nruns,1);       % -; amplitude of fourier components, scaled
hphyseq = cell(nruns,1);    % m; Profile of the sandbank in equilibrium (in physical space);
hcrest = nan(nruns,1);      % m; depth of the top of the sandbank in equilibrium
htrough= nan(nruns,1);      % m; depth of the trough of the sandbank in equilibrium
hrel = nan(nruns,1);        % -; relative height of the sandbank in equilibrium
Asymmetry = nan(nruns,1);   % -; Asymmetry of the sandbank in equilibrium
cmig = nan(nruns,1);        % m/yr; Migration rate of the sandbank in equilibrium
EQstate = nan(nruns,1);     % true/false; states whether an equilibrium is reached before tmormax

nvec = 1:nruns;

%parpool(6) %uncomment for parallel computing
%parfor n = nvec
for n = nvec
    [Lfgmdum,thetafgmdum,cfgmdum,hhatdum,hphyssave,EQstatedum] =...
        Bank_w_rock_func(Casename{n},Hstar(n),Hrockstar(n),drockstar(n),hinit(n),Ldomstar(n),topo_mode(n),domfac(n),lat(n),semastar(n,:),Uwstar(n),sigmastar(n),cd(n),...
        lambdastartilde(n),alphasstar(n),gammadeposition(n),P(n),jdt(n),Mnonlin(n),dtmor(n),tmormax(n),EQcrit(n),Vnl0crit(n),Saveswitch(n));

    Lfgm(n) = Lfgmdum; %Lfgm is in metres
    thetafgm(n) = thetafgmdum;
    cfgm(n) = cfgmdum;
    hhat{n} = hhatdum;
    hphyseq{n} = hphyssave(end,:);
    EQstate(n) = EQstatedum;
    disp(['Simulation ' num2str(n) 'has just finished.'])
end

%delete(gcp('nocreate')); % end parallel computing

%% writeresults
writedata = [Lfgm, thetafgm, cfgm, EQstate];
writematrix(writedata,'BwR_results.csv')





