%{
  Idealised numerical model for equilibrium shapes of tidal sandbanks under sediment-scarce conditions (1D)
  Copyright (C) 2023  Thomas van Veelen. Email: t.j.vanveelen@utwente.nl

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%}

%% Bank with Rock - fastest growing mode module
% This module identifies the fastest growing mode (fgm) based on tidal
% conditions and mean water depth
%
% Input: topo_mode, M, hinit, h0, DALwdw, domfac, loadfile
% Output: Lfgm, thetafgm, kxfgm, kyfgm
%
%
% Output:
% Lfgm - Normalized wavelength of the fastest growing mode
% thetafgm - Angle of the fastest growing mode
% kfgm - Normalized wave number of the fastest growing mode
% cfgm - Normalized migration speed og the fastest growing mode
function [Lfgm, thetafgm, kfgm, cfgm,Lfgmstar,kfgmstar,omega] = Bank_fgm(incvec,k,kstar,sema,ecc,pha,P,t,Tmoryear,Uw,mwind,h0,r,f,gammarel,jdt,lambda,PlotFlow,PlotLinOmega,DispFgm)

% initialisation
pvec = -P:P;                            % -; tool for truncation
Mlin = length(k);

inc = [incvec incvec incvec];
inc = inc / 180 * pi;                   % -; Inclination in radians

li = length(incvec);
omega = nan(Mlin,li);
cmigrationstar = nan(Mlin,li);


for th = 1:length(incvec)
    %% Basic flow
    [U0,V0] = BasicFlow(sema,ecc,inc(th,:),pha,P,t,PlotFlow);
    
    % wind waves
    Uw0 = Uw;
    
    % suspended transport
    Ce0 = conv(U0,U0,'same') + conv(V0,V0,'same');
    Ce0(P+1) = Ce0(P+1) + 0.5*Uw0^2;
    C0 = Ce0./(1+1i*pvec.'/gammarel);
    
    %% Linear model
    U1pmdivh1 = -U0/h0;
    Bm = -f*U1pmdivh1 + r/h0^2*V0;
    
    % for each wave mode m
    for m = 1:Mlin
        Aneg2 = sparse(diag((1i*k(m)*U0(P-1))*ones((2*P-1),1),2));    % Sparse matrix to save memory
        Aneg1 = sparse(diag((1i*k(m)*U0(P))*ones((2*P),1),1));
        Azero = sparse(diag((1i*k(m)*U0(P+1))*ones((2*P+1),1)));
        Apos1 = sparse(diag((1i*k(m)*U0(P+2))*ones((2*P),1),-1));
        Apos2 = sparse(diag((1i*k(m)*U0(P+3))*ones((2*P-1),1),-2));
        delta = sparse(diag((r/h0)*ones((2*P+1),1)));
        timeadd = jdt * sparse(diag(1i*pvec));
        Lm = Aneg2 + Aneg1 + Azero + Apos1 + Apos2 + delta + timeadd;
        
        V1pmdivh1 = Lm\Bm;
        
        % wind waves
        Uw1divh1 = -mwind*Uw;
        
        % suspended sediment transport
        Ce1pmdivh1 = 2*(conv(U0,U1pmdivh1,'same') + conv(V0,V1pmdivh1,'same'));
        Ce1pmdivh1(P+1) = Ce1pmdivh1(P+1) + 0.5*Uw0*Uw1divh1;
        
        cBm = -1i*k(m)*conv(C0,U1pmdivh1,'same') + gammarel*Ce1pmdivh1;
        cdelta = sparse(diag(gammarel*ones((2*P+1),1)));
        cLm = Aneg2 + Aneg1 + Azero + Apos1 + Apos2 + cdelta + timeadd; % reusing components from the hydrodynamic matrix.
        
        C1pmdivh1 = cLm\cBm;
        
        % growth rates
        omegam =  (gammarel* (Ce1pmdivh1(P+1)-C1pmdivh1(P+1)) - k(m)^2*lambda*Ce0(P+1));
        
        omega(m,th) = omegam;
        
        cmigrationstar(m,th) = -imag(omegam)/kstar(m)/Tmoryear;
    end
end

omegamax = max(max(real(omega)));
[ikfgm,ithetafgm]=find(real(omega)==omegamax,1,'last');
kfgmstar = kstar(ikfgm);
kfgm = k(ikfgm);
Lfgmstar = 2*pi./kfgmstar;
Lfgm = 2*pi./kfgm;
thetafgm = incvec(ithetafgm);
cfgmstar = cmigrationstar(ikfgm,ithetafgm);
cfgm = cfgmstar*Tmoryear*kfgmstar/kfgm;

if abs(cfgm)<eps
    cfgm = 0;
end
if DispFgm
    fprintf('L_{fgm} is %4.0f metres. theta_{fgm} is %2.1f degrees. \n',[Lfgmstar thetafgm])
end

kxstar = (sind(incvec)*kstar);
kystar = -(cosd(incvec)*kstar);

% plot linear results
if PlotLinOmega
    figure
    subplot(1,2,1)
    pcolor(kxstar,kystar,real(omega).'), hold on
    shading interp
    contour(kxstar,kystar,real(omega).',0:0.1:1,'k')
    xlabel('k_x [m^{-1}]')
    ylabel('k_y [m^{-1}]')
    colorbar
    caxis([-1 1])
    
    subplot(1,2,2)
    pcolor(kxstar,kystar,cmigrationstar.'), hold on
    shading interp
    xlabel('k_x [m^{-1}]')
    ylabel('k_y [m^{-1}]')
    cb = colorbar;
    title(cb,'m/yr')
end



end