%{
  Idealised numerical model for equilibrium shapes of tidal sandbanks under sediment-scarce conditions (1D)
  Copyright (C) 2023  Thomas van Veelen. Email: t.j.vanveelen@utwente.nl

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%}

%% Sandbanks_with_nonerodible layers (rocks)
% This script aims to recreate the sandbank model by Roos et al., 2004
% with the addition of a non-erodible layer. The model is idealized, 1D,
% and aims to study morphodynamic evolution of sandbanks on top of
% non-erodible layers. This cript is a function file that covers a full
% simulation of a parameter set.
function [Lfgmstar,thetafgm,cfgm,hhatsave,hphyssave,EQstate] = ...
    Bank_w_rock_func(Casename,Hstar,Hrockstar,drockstar,hinit,Ldomstar,topo_mode,domfac,lat,semastar,Uwstar,sigmastar,cd,...
    lambdastartilde,alphasstar,gammadeposition,P,jdt,Mnonlin,dtmor,tmormax,EQcrit,Vnl0crit,Saveswitch)

%% Input

%Numerical input
Ntime = 64;                         % Number of points within tidal cycle for tidal averaging
savefreq = 0.5;                     % frequency of saving hydrodynamic data per save step.
Vnl0maxit = 100;                     % max number of iterations in deriving Vnl0

% physical input
ecc = [0 0 0];                      % -; eccentricity for M0 (meaningless), M2, M4
pha = [0 0 0];                      % degree; phase for M0 (meaningless), M2 (meaningless), M4
mwind = 1;                          % -; wind stirring factor
Omegastar = 7.292e-5;               % rad/s; Angular frequency of the earth's rotation
gstar = 9.81;                       % m/s^2; gravitational acceleration
h0 = 1;                             % scaled average water depth

% Sediment transport properties
beta = 3;                           % -; power relation velocity-transport
epor = 0.4;                         % -; porosity

% linear domain parameters
kstar = 0:0.000025:0.0025;      % for finding fgm
incvec = (0:0.1:90).';          % inclination vector for M2-tide for finding fgm

% Computed quantities
Ustar = sum(abs(semastar(:)));          % m/s; Velocity scale
fstar = 2*Omegastar*sind(lat);          % 1/s; Coriolis parameter
rstar = cd*Ustar*8/3/pi;                     % m/s; Bottom friction parameter, 8/3pi * Ustar removed
Lstar = Ustar/sigmastar;                % m; Tidal excursion length
Tmorstar = (1-epor)*Hstar*Lstar/alphasstar/Ustar^beta;       % s; Morphological timescale
Tmoryear = Tmorstar/3600/24/365.25;     % year; Morphological timescale
lambdastar = lambdastartilde * Ustar;   % m/s; slope coefficient

% Time vectors
tstar = (0:(Ntime-1))*2*pi/sigmastar/Ntime;     % t over tidal cycle
tmorvec = 0:dtmor:tmormax;                      % vector for plotting
nsave = length(0:savefreq:tmormax);             % number of times that data is saved

% Numerical help variables
pha = pha / 180 * pi;                   % -; Phase in radians

% Saving
Savenamepre = Casename;

% set path to subfunctions
addpath('subfunctions')

%% Scaling
[Hrock,drock,t,f,r,sema,Uw,~,~,~,k,lambda,gammarel] = VariableScaling_rock(Hstar,Lstar,Hrockstar,drockstar,sigmastar,tstar,...
    Tmorstar,fstar,rstar,semastar,Ustar,Uwstar,gstar,kstar,Ldomstar,lambdastar,gammadeposition);

%% Find Fastest Growing Mode (fgm);

[~, thetafgm, ~, cfgm,Lfgmstar,~] = Bank_fgm(incvec,k,kstar,sema,ecc,pha,P,t,Tmoryear,Uw,mwind,h0,r,f,gammarel,jdt,lambda,0,0,1);

%% Set domain parameters
% Domain length, if function of fgm
Ldomstar = Lfgmstar;
if topo_mode == 2
    Ldomstar = domfac * Lfgmstar;
end 


% Numerical parameters
M = Mnonlin;
mvec = -M:M;

Mphys = 2*Mnonlin;
xstar = (0:(Mphys-1))*Lfgmstar/Mphys; 
k1star = 2*pi/Ldomstar;
kstar = mvec*k1star;
k = kstar*Lstar;

% topography
loadfile = ['hcntd_' Savenamepre '.mat']; % only used in topo-mode 9

[hhat, hphys] = BwR_topo(topo_mode, M, hinit, h0, domfac, loadfile);


%% Compute the linear growth rates of the spatial modes and basic flow
% relevant for the semi-implicit time-stepping
[~,~,~,~,~,~,omegafornl] = Bank_fgm(thetafgm,k,kstar,sema,ecc,pha,P,t,Tmoryear,Uw,mwind,h0,r,f,gammarel,jdt,lambda,0,0,1);

omegafornl = omegafornl.';

% basic flow
thfgmrad = thetafgm / 180 * pi;
[U0,V0] = BasicFlow(sema,ecc,thfgmrad,pha,P,t,0);

%% nonlinear behaviour
[hhatsave, hphyssave, Bmnlsave, cmignlsave, Chisave, Unlsave, Vnlsave, Cenlsave, Cnlsave, mulimsave, hrms, Gamma, ntsave, tmorvec ,~, EQstate] = BwR_MorphoEvo(M,P,tmorvec,...
    hhat,hphys,h0,Hrock,drock,k,U0,V0,Uw,mwind,f,r,jdt,gammarel,lambda,omegafornl,EQcrit,Vnl0crit,Vnl0maxit,nsave,savefreq,Saveswitch,Savenamepre,hinit,Hstar,Hrockstar,drockstar,lat,xstar,Lstar,Tmoryear);

%% Saving 
if Saveswitch
Savename = sprintf([Savenamepre '_hi%1.3f_H%2.1f_Hr%2.1f_dr%2.2f_lat%d_M%d_dt%d.mat'],hinit,Hstar,Hrockstar,drockstar,lat,Mnonlin,round(1/dtmor));

save(Savename,'hhatsave','hphyssave','Bmnlsave','cmignlsave','Chisave','Unlsave','Vnlsave','Cenlsave','Cnlsave','mulimsave','hrms','Gamma','ntsave','xstar','dtmor','Hstar','Hrockstar','drockstar','M','tmorvec','EQstate','Lstar','Tmoryear')
end
%}