%{
  Idealised numerical model for equilibrium shapes of tidal sandbanks under sediment-scarce conditions (1D)
  Copyright (C) 2023  Thomas van Veelen. Email: t.j.vanveelen@utwente.nl

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%}

function [U0,V0,BscFlow] = BasicFlow(sema,ecc,inc,pha,P,t,PlotFlow)
%% Subfunction of the SandExtractionModel. Herein the basic flow (u0,v0) is solved.
%  The basic flow depends on the tidal amplitude (sema), eccentricity,
%  inclination and phase. The output is the basic flow and the Fourier
%  components of the basic flow

U0 = zeros(2*P+1,1);                                                        % Fourier components of the tidal flow (x-dir)
V0 = zeros(2*P+1,1);                                                        % Fourier components of the tidal flow (y-dir)
pvec = -P:P;                                                                % Helpvector containing the time components

r_V0 = 1/4 * sema .* (exp(1i*inc) + exp(-1i*inc));                            % Determine the real amplitude of the U0 components.
r_U0 = 1/(4*1i) * sema .* (exp(1i*inc) - exp(-1i*inc));                        % Determine the real amplitude of the V0 components.
q_V0 = ecc .* r_V0;                                                         % Determine the complex amplitude of the U0 components.
q_U0 = -ecc .* r_U0;                                                        % Determine the complex amplitude of the V0 components.

U0(P+1) = 2 * r_U0(1);                                                      % Determine U0 and V0 vectors
U0(P+2:P+3) = (r_U0(2:3) + q_U0(2:3)*1i) .* exp(-1i*pha(2:3));
U0(P:-1:P-1) = conj(U0(P+2:P+3));                                           % Complex conjugate; this will result in cos/sin function
V0(P+1) = 2 * r_V0(1);
V0(P+2:P+3) = (r_V0(2:3) + q_V0(2:3)*1i) .* exp(-1i*pha(2:3));
V0(P:-1:P-1) = conj(V0(P+2:P+3));

% filter out rounding errors
U0 = (abs(U0) > eps).*U0;
V0 = (abs(V0) > eps).*V0;

u0 = U0.' * exp(1i*pvec'*t);
v0 = V0.' * exp(1i*pvec'*t);
absu0 = (u0.^2+v0.^2).^0.5;                                                 % Absolute flow velocity

if(PlotFlow==1)
    figure
    subplot(1,3,1)
    plot(u0,v0,'m')
    hold on;
    plot([0 1]*u0(1),[0 1]*v0(1),'g--')
    axis(1.5*[-1 1 -1 1])
    axis square
    title('(a) tidal ellipse')
    xlabel('u_0 [-]')
    ylabel('v_0 [-]')
    
    
    subplot(1,3,2)
    plot(t/(2*pi),u0,'b')
    hold on;
    plot(t/(2*pi),v0,'r')
    axis([0 1 -1.5 1.5])
    axis square
    title('(b) flow components')
    xlabel('t [-]')
    ylabel('u_0, v_0 [-]')
    legend('u0','v0')
    
    subplot(1,3,3)
    plot(t/(2*pi),absu0,'k')
    axis([0 1 0 1.5])
    axis square
    title('(c) flow velocity magnitude')
    xlabel('t [-]')
    ylabel('|(u_0,v_0)| [-]')
end

BscFlow = [u0' v0' absu0'];
end