%{
  Idealised numerical model for equilibrium shapes of tidal sandbanks under sediment-scarce conditions (1D)
  Copyright (C) 2023  Thomas van Veelen. Email: t.j.vanveelen@utwente.nl

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%}

%% Bank with Rock - Morphodynamic evolution module
% This module computes the morphodynamic evolution of a tidal sandbank. It
% continues until an equilibrium is reached. The computations follows the
% morphodynamic loops and consider nonlinear dynamics
%
%
function [hhatsave, hphyssave, Bmnlsave, cmignlsave, Chisave, Unlsave, Vnlsave, Cenlsave, Cnlsave, mulimsave,  hrms, Gamma, ntsave, tmorvec,it,EQstate]...
    = BwR_MorphoEvo(M,P,tmorvec,hhat,hphys,h0,Hrock,drock,k,U0,V0,Uw,mwind,f,r,jdt,gammarel,lambda,omegafornl,EQcrit,Vnl0crit,Vnl0maxit,...
    nsave,savefreq,Saveswitch,Savenamepre,hinit,Hstar,Hrockstar,drockstar,lat,xstar,Lstar,Tmoryear)

%% Initialisation 
% Numerical parameters
Mtot = 2*M+1;
mvec = -M:M;
Mphys = 2*M;
Ptot = 2*P+1;
pvec = -P:P;
PMtot = Ptot*Mtot;
ntmor = length(tmorvec)-1;                   % -; number of morphological time steps
dtbase = tmorvec(2)-tmorvec(1);
mulim = ones(1,Mphys);

% Prepare matrices
hphyssave = nan(ntmor+1,Mphys);
hhatsave = nan(ntmor+1,Mtot);

Bmnlsave = nan(ntmor,Mtot);
cmignlsave = nan(ntmor,Mtot);

Chisave = nan(Ptot,nsave);
Unlsave = nan(Ptot,Mtot,nsave);
Vnlsave = nan(Ptot,Mtot,nsave);
Cenlsave = nan(Ptot,Mtot,nsave);
Cnlsave = nan(Ptot,Mtot,nsave);
mulimsave = nan(nsave,Mphys);
hrms = nan(nsave,1);
Gamma = nan(nsave,1);
ntsave = nan(nsave,2); % column 1: number, col 2: time
savecounter = 1;

hhatsave(1,:) = hhat;
hphyssave(1,:) = hphys;

hmax = max(hphys);
bufferevent = 1; % event switch for crossing the buffer layer for the first time;
lbe = 1;
lbeafterevent = 0.4;

%% basic flow
% basic flow components of the fgm (basic flow recomputed at linear for
% fgm & do not change with bank growth)
Px = (jdt*1i*pvec.'+r/h0).*U0 - f*V0;
Py = (jdt*1i*pvec.'+r/h0).*V0 + f*U0;

EQstate = 0; % trigger for equilibrium state. It moves to 1 when growth rate of all modes is smaller than equilibrium criterion
EQcounter = 0; % counter for number of times that equilibrium condition is satisfied consecutively  
EQtargetcount = 10;
it = 1; % timestep
while and(EQstate == 0, it <= ntmor)
    
    % Hydrodynamic  matrix preparations
    sclm = sparse(zeros(PMtot));
    
    pvecmat = repmat(pvec,[1 Mtot]);
    kvecmat = repelem(k,Ptot);
    
    fric = sparse(diag(r*ones(PMtot,1))); % Friction contribution for Vnl
    
    for m =1:Mtot %build spatial convolution components for Vnl
        mm = mvec(m);
        dg = sparse(diag(hhat(m)*ones((Mtot-abs(mm))*Ptot,1),-mm*Ptot));
        sclm = sclm + dg;
    end
    sclm = jdt * 1i*pvecmat.'.*sclm;
    
    
    % solve chi
    divh0 = mean(1./hphys);
    divhsq0 = mean(1./hphys.^2);
    
    hLm = zeros(Mtot);         % spatial convolution matrix
    divhBm = zeros(Mtot,1);    % Right hand side in the solution of 1/h
    
    divhBm(M+1) = 1;
    
    for m = 1:Mtot % set up the spatial convolution matrix which is independent of temporal mode
        mm = mvec(m);
        dg = diag(hhat(m)*ones(Mtot-abs(mm),1),-mm);
        hLm = hLm +dg;
    end
    
    divhnl = (hLm\divhBm).';
    
    % ############################################
    % hydrodynamic solution, iterative approach for V0
    Vnl0err = Inf;
    Vnl0it = 0;  % number of iterations in Vnl0

    while Vnl0err > Vnl0crit
        Vnl0it = Vnl0it+1;
        
        % Hydrodynamic  matrix preparations
        vLm = sparse(zeros(PMtot));
        vBm = nan(PMtot,1);
        tclm = sparse(zeros(PMtot));
        
        if f == 0
            Vnl0 = 0; % iterations are irrelevant as f = 0
        else
            % Solve Chi0 and v00 simultaneously v00 can be solved without
            % iterations
            vcsm = [Py(P+1)*h0; Px(P+1)];
            vclm = [f r; r*divhsq0 -f];
            
            vcsol = vclm\vcsm;
            if and(Vnl0it == 1,it == 1)
                Vnl0 = V0; % use basic flow if there is no prior iteration, or timestep
                
                
            else
                Vnl0 = Vnl0new; % this is either a prior iteration or previous timestep.
            end
            Vnl0(P+1) = vcsol(2); % use analytical mean flow solution
        end
        
        Chi = (Px + f*Vnl0)./(jdt*1i*pvec.'*divh0 + r*divhsq0); 
        
        % solve u via definition of Chi
        Unl = Chi*divhnl;
        
        % solve v via spatiotemporal convolution
        for p = 1:Ptot
            for m = 1:Mtot
                if m == (M+1)
                    vBm(Ptot*(m-1)+p) = Py(p)*hhat(m) - f*Chi(p);
                else
                    vBm(Ptot*(m-1)+p) = Py(p)*hhat(m);
                end
            end
        end
        
        
        for p = 1:Ptot %build temporal convolution components
            pp = pvec(p);
            pkeep = repmat([zeros(1,abs(pp)) ones(1,Ptot-abs(pp))],[1 Mtot]);
            dgdum = Chi(p)*pkeep;
            dgdum(1:abs(pp)) = [];
            dg = sparse(diag(dgdum,-pp));
            tclm = tclm + dg;
        end
        tclm = 1i*kvecmat.'.*tclm;
        
        vLm = vLm + fric + tclm + sclm;
        
        Vnl = vLm\vBm;
        
        Vnl = reshape(Vnl,[Ptot Mtot]);
        
        % calculate error in Vnl0
        Vnl0new = Vnl(:,M+1);
        if f == 0
            Vnl0err = 0;
        else
            Vnl0err = max(abs(Vnl0new - Vnl0));
        end
        if Vnl0it == Vnl0maxit
            disp('Warning: no convergence in the spatial average of V within the maximum number of allowed iterations.')
            return
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    

    % Suspended load transport

    Uphys = nan(Ptot,Mphys);
    Vphys = nan(Ptot,Mphys);

    % via physical method
    for p = 1:Ptot
        Uphys(p,:) = ifft(ifftshift(Unl(p,1:Mphys)* Mphys));
        Vphys(p,:) = ifft(ifftshift(Vnl(p,1:Mphys)* Mphys));
    end
    
    dhhatdx = 1i*k.*hhat;
    
    uwphys = Uw*hphys.^(-mwind);
    uw2phys = uwphys.^2;
    
    Cephysbase = nan(size(Uphys));
    for m = 1:(Mtot-1)
        Cephysbase(:,m) = conv(Uphys(:,m),Uphys(:,m),'same') + conv(Vphys(:,m),Vphys(:,m),'same');
        Cephysbase(P+1,m) = Cephysbase(P+1,m) + 0.5*uw2phys(m);
    end

    % Define limiter (physical space)
    for m = 1:Mphys
        hm = hphys(m);
        if hm > (Hrock-drock)
            if hm > Hrock
                mulim(m) = 0;
            else
                hcalc = 1 + (hm - Hrock)/drock;
                mulim(m) = 1 - 10*hcalc.^3 + 15*hcalc.^4 -6*hcalc.^5;
            end
        end
    end

    Cephys = mulim.*Cephysbase;
    Celim = zeros(Ptot,Mtot);

    for p = 1:Ptot
        Celim(p,1:Mphys) = fftshift(fft(Cephys(p,1:Mphys)/Mphys));
    end
    Celim(:,end)=Celim(:,1);
    
    % suspended load
    % entrainment concentration was calculated via bedload
    
    cLm = sparse(zeros(PMtot));
    stlm = sparse(zeros(PMtot));
    cBm = gammarel * Celim(:);
    
    cfric = sparse(diag(gammarel*ones(1,PMtot)));
    cdt = jdt*sparse(diag(1i*pvecmat.*ones(1,PMtot)));
    
    Unlvec = Unl(:);
    pmvec = (-M*Ptot-P):(M*Ptot+P);
    
    for pm = 1:(PMtot)
        pmpm = pmvec(pm);
        dg = sparse(diag(Unlvec(pm)*ones(PMtot-abs(pmpm),1),-pmpm));
        stlm = stlm + dg;
    end
    
    stlm = 1i*kvecmat.'.*stlm;
    
    cLm = cLm + cfric + cdt +stlm;
    
    Cnl = cLm\cBm;
    
    Cnl = reshape(Cnl,[Ptot Mtot]);
    
    % C in physical space
    Cphys = nan(Ptot,Mphys);
   
    % via physical method
    for p = 1:Ptot
        Cphys(p,:) = ifft(ifftshift(Cnl(p,1:Mphys)*Mphys));
    end
    
   
    
    %% Bed evolution
    
    Ce0dhdxnl = conv(Celim(P+1,:),dhhatdx,'same');
    dCe0dhdxdx = 1i*lambda*k.*Ce0dhdxnl;
    
    Bmnl = (gammarel*(Celim(P+1,:)-Cnl(P+1,:)) + dCe0dhdxdx);
    Bmnl(abs(Bmnl)<eps) = 0;
    
    cmignl  = -Bmnl./(1i*k.*hhat);
     
    % Dynamic time step correction
    % growth rate limiter
    Bmax = 0.01;
    if max(abs(Bmnl)) > Bmax
        lgr = Bmax/ max(abs(Bmnl));
        if lgr < 0.02
            lgr = 0.02;
        end
    else
        lgr = 1;
    end
    
    % depth limiter
    if and(hmax >= (Hrock-drock), hmax <= (Hrock-0.5*drock)) % apply limiter in the top half of the buffer layer.
        hcalc = (hmax - (Hrock-drock))/(0.5*drock);
        ldp = 0.1 + 0.9 * hcalc;
    else
        ldp = 1;
    end
        
    dtmor = dtbase * lgr * ldp * lbe;
    
    % Time stepping
    hhatnew = hhat + Bmnl./(1/dtmor - omegafornl);
    hphysnew = real(ifft(ifftshift(hhatnew(1:Mphys)*Mphys)));
    
    hmaxnew = max(hphysnew);
    
    
    % event checker
    if and(bufferevent,hmaxnew >= (Hrock-drock))
        hdiff = hmaxnew - hmax;
        hrockdiff = Hrock - drock - hmax;
        dtmor = dtmor * hrockdiff/hdiff;
        bufferevent = 0;
        lbe = lbeafterevent;
        
        hhat = hhat + Bmnl./(1/dtmor - omegafornl);
        hphys = real(ifft(ifftshift(hhat(1:Mphys)*Mphys)));
        
        hmax = Hrock-drock;
    elseif hmaxnew >= Hrock
        hdiff = hmaxnew - hmax;
        hrockdiff = Hrock - hmax;
        dtmor = dtmor * hrockdiff/hdiff/2; % ensuring that new deepest point is halfway current deepest point and non-erodible layer;
        
        hhat = hhat + Bmnl./(1/dtmor - omegafornl);
        hphys = real(ifft(ifftshift(hhat(1:Mphys)*Mphys)));
        
        hmax = max(hphys);
    else
        hhat = hhatnew;
        hphys = hphysnew;
        hmax = hmaxnew;
    end
    
    tmorvec(it+1) = tmorvec(it) + dtmor;
    
    % saving
    Bmnlsave(it,:) = Bmnl;
    cmignlsave(it,:) = cmignl;
    hhatsave(it+1,:) = hhat;
    hphyssave(it+1,:) = hphys;
    
    if or(mod(it*dtbase,savefreq) == 0,it==1)
        isave = savecounter;
        Chisave(:,isave) = Chi;
        Unlsave(:,:,isave) = Unl;
        Vnlsave(:,:,isave) = Vnl;
        Cenlsave(:,:,isave)= Celim;
        Cnlsave(:,:,isave)= Cnl;
        mulimsave(isave,:) = mulim;
        ntsave(isave,1) = isave;
        ntsave(isave,2) = tmorvec(it+1);
        
        hrms(isave) = sqrt(sum(abs(hhat).^2)- h0.^2);
        if it > 1
            Gamma(isave) = 1/hrms(isave)^2 * (0.5*hrms(isave)^2 - 0.5*hrms(isave-1)^2)/ (ntsave(isave,2) - ntsave(isave-1,2));
        end
        
        savecounter = savecounter +1;
        
        % check for equilibrium
        if abs(Gamma(isave)) < EQcrit
            EQcounter = EQcounter + 1;
            if EQcounter == EQtargetcount
                EQstate = 1;
            end
        else
            EQcounter = 0;
        end
    end
    
    if mod(it*dtbase,1) == 0
        disp(tmorvec(it+1))
        
        if Saveswitch
            dtmor = dtbase;
            
            Savename = sprintf([Savenamepre '_hi%1.3f_H%2.1f_Hr%2.1f_dr%2.2f_lat%d_M%d_dt%d.mat'],hinit,Hstar,Hrockstar,drockstar,lat,M,round(1/dtmor));
            
            save(Savename,'hhatsave','hphyssave','Bmnlsave','cmignlsave','Chisave','Unlsave','Vnlsave','Cenlsave','Cnlsave','mulimsave','hrms','Gamma','ntsave','xstar','dtmor','Hstar','Hrockstar','drockstar','M','tmorvec','EQstate','Lstar','Tmoryear')
        end
    end
    
    it =it+1;
end


end