%{
  Idealised numerical model for equilibrium shapes of tidal sandbanks under sediment-scarce conditions (1D)
  Copyright (C) 2023  Thomas van Veelen. Email: t.j.vanveelen@utwente.nl

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%}

%% Bank with Rock - topography module
% This module generates topographies for sandbanks with a rocky
% substrate. It generates topogrpahies in spectral and physical space.
% Input: topo_mode, M, hinit, h0, DALwdw, domfac, loadfile 
% Output: hhat, hphys, Ldom
% 
% Topo modes
% 1: FGM - sinusoidal
% 2: FGM - long domain
% 3: Random bed - long domain
% 9: Load domain
%
% M - number of modes
% hinit - normalized initial crest elevation
% h0 - normalized average water depth.
% domfac - length factor. Domain length is domfac times fgm wave length.
% Must be a positive integer. Only used in topo_mode 2.
% loadfile - name of the file that contains the intial topogrpahy. Format
% '.mat'. File must contain hhat and hphys.
%
function [hhat, hphys] = BwR_topo(topo_mode, M, hinit, h0, domfac, loadfile)

Mtot = 2*M+1;                       % Modes in spectral space

hhat = zeros(1,Mtot);
hphys = zeros(1,Mtot-1);

if topo_mode == 1
    hhat(M+1) = h0;
    hhat([M M+2]) = hinit/2;
    
    hphys = ifft(ifftshift(hhat(1:(Mtot-1))*(Mtot-1)));
    
elseif topo_mode == 2
    hhat(M+1) = h0;
    hhat(M+1+domfac*[-1 1]) = hinit/2;
    
    hphys = ifft(ifftshift(hhat(1:(Mtot-1))*(Mtot-1)));
    
elseif topo_mode == 3

    hrand = 2*hinit*rand(size(hphys));
    hrand = hrand - mean(hrand);
    hphys = h0 + hrand;
    
    hhat(1:Mtot-1) = fftshift(fft(hphys(1:(Mtot-1))/(Mtot-1)));
    hhat(Mtot) = hhat(1);
    
    
elseif topo_mode == 9
    
    load(loadfile,'hhat','hphys')

end 